import logging

from pyVmomi import vim
from pyVim.connect import SmartConnectNoSSL, Disconnect

from ruxit.api.base_plugin import RemoteBasePlugin
from ruxit.api.exceptions import ConfigException

logger = logging.getLogger(__name__)


def find_objects(content, vimtype):
    """
    Returns a list of objects of vimtype
    :param content: pyVmomi.VmomiSupport.vim.ServiceInstanceContent,
    usually obtained from service_instance.RetrieveContent()
    :param vimtype: VIM Data type, example: vim.Datastore
    :return: List of objects found
    """
    return [
        item
        for item in content.viewManager.CreateContainerView(
            content.rootFolder, [vimtype], recursive=True
        ).view
    ]


class VMwareVsanExtension(RemoteBasePlugin):
    def query(self, **kwargs) -> None:
        config = kwargs.get("config")
        host = config.get("vsphere_host")
        port = config.get("vsphere_port")
        if port is None or not port:
            port = 443
        username = config.get("vsphere_username")
        password = config.get("vsphere_password")

        group_name = config.get("group")
        if not group_name:
            group_name = "VMware vSan"

        device_name = config.get("device")
        if not device_name:
            device_name = f"vSan@{host}"

        group = self.topology_builder.create_group(group_name, group_name)
        device = group.create_device(device_name)

        if not host or not username or not password:
            raise ConfigException("Host, Username and Password are mandatory")

        service_instance = SmartConnectNoSSL(
            host=host, user=username, pwd=password, port=port
        )
        content = service_instance.RetrieveContent()

        datastores = find_objects(content, vim.Datastore)
        for ds in datastores:
            # We only care about VSan DataStores
            if ds.summary.type == "vsan":
                name = ds.summary.name
                logger.info(f"Processing datastore: '{name}'")

                device.report_property("vSan Datastores", f"{len(datastores)}")
                device.absolute(
                    "capacity", ds.summary.capacity, dimensions={"Datastore": name}
                )
                device.absolute(
                    "free_space", ds.summary.freeSpace, dimensions={"Datastore": name}
                )
                device.absolute(
                    "used",
                    100 - 100 * ds.summary.freeSpace / ds.summary.capacity,
                    dimensions={"Datastore": name},
                )
                device.state_metric(
                    "status", ds.overallStatus.upper(), dimensions={"Datastore": name}
                )
                device.absolute("hosts", len(ds.host), dimensions={"Datastore": name})

                for host in ds.host:
                    device.state_metric(
                        "host_status",
                        host.overallStatus.upper(),
                        dimensions={"Datastore": name, "Host": host.name},
                    )

        Disconnect(service_instance)
